package winDlg;

# winDlg.pm
#
# [ldasm] Creates all required dialogs
#

use strict;

use winMain;
use winFunc;
use misc;
use config;

#-------
# Export

use Exporter;
use vars qw(@ISA @EXPORT
            %SearchText %SearchResult
            %GotoCodeLoc %FuncImport %FuncExport %FuncInternal %StringRefs
            %HelpMe %About
            %Progress);
@ISA = qw(Exporter);
@EXPORT = qw(&CreateAllDialogs
            %SearchText %SearchResult %GotoCodeLoc %FuncImport %FuncExport
            %FuncInternal %StringRefs
            %HelpMe %About
            %Progress);

#-----------------
# Dialog-variables

%SearchText = (
   btn  => [
      'Cancel','Search'],
   txt    => '',
   case   => 0,
   regexp => 0,
   dir    => 'down',
   reswin => 0
);
%SearchResult = (
   btn => {
      'Goto' => sub { winFunc::SearchResultGoto(); }
   },
   matches => 0
);
%GotoCodeLoc = (
   btn  => [
      'Cancel','OK'],
   ofs  => 0
);
my %RefsButtons = (
  'Copy All'  => [\&winFunc::ReferenceCopy,'all'],
  'Copy View' => [\&winFunc::ReferenceCopy,'view']
);

#-------------
# _DlgSearch()
#
# Dialog for "search-Text"

sub _DlgSearch()
{
   $SearchText{dlg} = $winMain::win->DialogBox(
      -title   => 'Search',
      -buttons => [@{$SearchText{btn}}],
      -default_button => $SearchText{btn}->[1]
   );
   my $sinput = $SearchText{dlg}->add('Frame')->pack(
      -side => 'top'
   );
   my $sopts1 = $SearchText{dlg}->add('Frame')->pack(
      -side => 'left'
   );
   my $sopts2 = $SearchText{dlg}->add('Frame')->pack(
      -side   => 'right'
   );
   $sinput->Label(
      -text => 'Search for:'
   )->pack(
      -side => 'left',
      -padx => 6
   );
   $sinput->Entry(
      -textvariable => \$SearchText{txt},
      -width        => 30,
      -bg           => 'white'
   )->pack(
      -side => 'right'
   );
   $sopts1->Checkbutton(
      -text => 'Case-sensitive',
      -variable => \$SearchText{case}
   )->pack(
      -side   => 'top',
      -anchor => 'w'
   );
   $sopts1->Checkbutton(
      -text => 'Regular expression',
      -variable => \$SearchText{regexp}
   )->pack(
      -side   => 'top',
      -anchor => 'w'
   );
   $sopts1->Checkbutton(
      -text     => 'Use result-window',
      -variable => \$SearchText{reswin}
   )->pack(
      -side   => 'top',
      -anchor => 'w'
   );
   my $sdl = $sopts2->Label(
      -relief => 'groove',
   )->pack(
      -side => 'left'
   );
   $sdl->Label(
      -text => 'Search-direction:'
   )->pack(
      -side => 'top'
   );
   $sdl->Radiobutton(
      -text     => 'Up',
      -variable => \$SearchText{dir},
      -value    => 'up'
   )->pack(
      -side => 'left'
      );
   $sdl->Radiobutton(
      -text     => 'Down',
      -variable => \$SearchText{dir},
      -value    => 'down'
   )->pack(
      -side => 'left'
   );
}

#-------------------
# _DlgSearchResult()
#
# Dialog for "SearchResult

sub _DlgSearchResult()
{
   $SearchResult{top} = ClientWin(
      -title   => "Result",
      -buttons => $SearchResult{btn}
   );
   my $flframe = $SearchResult{top}->Frame()->pack(
      -side   => 'top',
      -anchor => 'w'
   );
   $flframe->Label(
      -textvariable => \$SearchResult{matches}
   )->pack();
   $SearchResult{listbox} = $flframe->ScrlListbox(
      -width      => 10,
      -height     => 15,
      -background => 'white'
   )->pack();

   # Mouse-binding
   $SearchResult{listbox}->bind('<Double-1>',\&winFunc::SearchResultGoto);
}

#------------------
# _DlgGotoCodeLog()
#
# Dialog for "GotoCodeLoc"

sub _DlgGotoCodeLog()
{
   $GotoCodeLoc{dlg} = $winMain::win->DialogBox(
      -title   => 'Select Offset',
      -buttons => [@{$GotoCodeLoc{btn}}],
      -default_button => $GotoCodeLoc{btn}->[1]
   );
   $GotoCodeLoc{dlg}->add('Label',
      -text => 'Goto Code Location'
   )->pack(
      -side => 'left',
      -padx => 6
   );
   $GotoCodeLoc{dlg}->add('Entry',
      -textvariable => \$GotoCodeLoc{ofs},
      -width        => 20,
      -bg           => 'white'
   )->pack(
      -side => 'left'
   );
}

#---------------------
# _DlgReference(\%$$$)
#
# Creates the toplevel for references
#
# in:  Hashref for toplevel, title, desc, ref-type
# out: -

sub _DlgReference(\%$$$)
{
   my ($hrt,$title,$dsc,$rtype) = @_;

   # Assign button-hash to ref-hash
   foreach (keys %RefsButtons) {
      $hrt->{btn}->{$_} = [@{$RefsButtons{$_}},$hrt];
   }

   # Dialog
   $hrt->{top} = ClientWin(
      -title   => "Alphabetical List of $title",
      -buttons => $hrt->{btn}
   );
   my $flframe = $hrt->{top}->Frame()->pack(
      -side   => 'top',
      -anchor => 'w'
   );
   $flframe->Label(
      -text => "To search disassembly for $dsc, double-click on text"
   )->pack();
   $hrt->{listbox} = $flframe->ScrlListbox(
      -width      =>  65,
      -height     => 15,
      -background => 'white'
   )->pack();
   $hrt->{type} = $rtype;

   # Mouse-binding
   $hrt->{listbox}->bind('<Double-1>' => sub { winFunc::JumpToReference($hrt); });
}

#-------------
# _DlgHelpMe()
#
# Help-Dialog - haha beginner

sub _DlgHelpMe()
{
   $HelpMe{top} = ClientWin(
      -title => 'Help (rtfm)'
   );
   print $HelpMe{top}->geometry('620x300');
   $HelpMe{top}->minsize(620,300);
   $HelpMe{top}->resizable('n','y');

   $HelpMe{txt} = $HelpMe{top}->ScrlText(
      -scrollbars => 'e',
      -wrap       => 'none',
      -background => 'white'
   )->pack(
      -expand => 1,
      -fill   => 'both'
   );

   # Insert help-file into text
   TextTags($HelpMe{txt},%config::FontHelp);
   my $fileFound = 1;
   open(HLPFILE,"< $config::RCPath/misc/help.txt") or $fileFound = 0;

   if ($fileFound) {
      while (my $line = <HLPFILE>) {
         # Comment ?
         next if ($line =~ /^#/);
         # Is type-desc
         if ($line =~ s/^\[([^\]]+)\]//) {
            my ($type,$spec) = split(/:/,$1);
            # Title
            if ($type eq 't') {
               $HelpMe{txt}->insert('insert',"\n$line\n\n",'title');
            # Section
            } elsif ($type eq 's') {
               $HelpMe{txt}->insert('insert',$line,'section');
            # Link
            } elsif ($type eq 'l') {
               $HelpMe{link}->{$spec} = $HelpMe{txt}->index('insert');
            # Anchor
            } elsif ($type eq 'a') {
               $HelpMe{anchor}->{$HelpMe{txt}->index('insert')} = $spec;
               $HelpMe{txt}->insert('insert',$line,'link');
            # Image
            } elsif ($type eq 'i') {
               $HelpMe{txt}->insert('insert',' 'x($HelpMe{txt}->tagCget('text','lmargin1') >>  3));
               my $img = $HelpMe{txt}->Pixmap(-file => "$config::RCPath/$spec.xpm");
               $HelpMe{txt}->imageCreate('insert',
                  -image => $img,
                  -align => 'baseline',
                  -padx  => 6,
                  -pady  => 0
               );
            # New paragraph
            } elsif ($type eq 'p') {
               $HelpMe{txt}->insert('insert',"\n");
            }
         # Normal text
         } else {
            $HelpMe{txt}->insert('insert',$line,'text');
         }
      }
      close(HLPFILE);
   } else {
      ErrBox("Unable to open the helpfile\n$config::RCPath/misc/help.txt");
   }
   $HelpMe{txt}->configure(-state => 'disabled'); # readonly
   $HelpMe{txt}->tagBind('link','<Button-1>',\&winFunc::HelpGotoAnchor);
}

#------------
# _DlgAbout()
#
# "About"-Dialog

sub _DlgAbout()
{
   $About{dlg} = $winMain::win->DialogBox(
      -title => 'About',
   );

   my $label = $About{dlg}->add('Label')->pack(
      -side => 'top');
   my $image = $label->Pixmap(-file => $config::RCPath.'/pics/logo.xpm');
   $label->configure(-image => $image);

   $About{dlg}->add('Label',
      -anchor => 'n',
      -relief => 'groove',
      -text   => "Version $config::VersionStr.$config::BuildStr\n"
                 ."Ravemax/Dextrose (ravemax\@dextrose.com)"
   )->pack(
      -side => 'bottom',
      -fill => 'x'
   );
}

#--------------
# _DlgProgress()
#
# Toplevel to display the progress

sub _DlgProgress()
{
   $Progress{top} = ClientWin(
      -title   => 'Progress...',
      -buttons => undef
   );
   $Progress{text} = $Progress{top}->ScrlText(
      -width       => 50,
      -height     => 8,
      -scrollbars => 'e',
      -bg         => 'gray'
   )->pack(
      -expand  => 1,
      -fill    => 'both'
   );
   TextTags($Progress{text},%config::FontProgress);
}

#-------------------
# CreateAllDialogs()
#
# Creates all dialogs

sub CreateAllDialogs
{
   _DlgSearch();
   _DlgSearchResult();
   _DlgGotoCodeLog();
   _DlgReference(%FuncImport,'Imported Functions','function','import');
   _DlgReference(%FuncExport,'Exported Functions','function','export');
   _DlgReference(%FuncInternal,'Internal Functions','function','internal');
   _DlgReference(%StringRefs,'String Data Items','string data','str');
   _DlgHelpMe();
   _DlgAbout();
   _DlgProgress();
}


1;
